#include "AttributeInfo.hpp"

#include "ByteClass.hpp"

const string AttributeInfo::Code = string("Code");
const string AttributeInfo::SourceFile = string("SourceFile");
const string AttributeInfo::LineNumberTable = string("LineNumberTable");
const string AttributeInfo::ConstantValue = string("ConstantValue");
const string AttributeInfo::Exceptions = string("Exceptions");
const string AttributeInfo::InnerClasses = string("InnerClasses");
const string AttributeInfo::Synthetic = string("Synthetic");
const string AttributeInfo::LocalVariableTable = string("LocalVariableTable");
const string AttributeInfo::Deprecated = string("Deprecated");

AttributeInfo::AttributeInfo(const u_int16_t& ani /*= 0*/,
                             const u_int32_t& l /*= 0*/,
                             u_int8_t* i /*= NULL*/)
  : attribute_name_index(ani),
    attribute_length(l),
    info(i) {
  }

AttributeInfo::~AttributeInfo() {
  if (info) {
    delete[] info;
    info = NULL;
  }
}

void AttributeInfo::setAttributeNameIndex(const u_int16_t& ni) {
  attribute_name_index = ni;
}

void AttributeInfo::setData(const u_int32_t& l, const u_int8_t* i) {
  if ((attribute_length != l) || !info) {
    // we have to realloc!
    attribute_length = l;
    if (info) {
      delete[] info;
    }
    info = new u_int8_t[attribute_length];
  }

  memcpy(info, i, attribute_length);
}

const u_int16_t& AttributeInfo::getAttributeNameIndex() const {
  return attribute_name_index;
}

const u_int32_t& AttributeInfo::getAttributeLength() const {
  return attribute_length;
}

ostream& AttributeInfo::toOStream(ostream& os) const {
  os << "AttributeInfo:"
     << "\tname_idx(" << getAttributeNameIndex() << ")"
     << "\tlength(" << getAttributeLength() << ")";

  return os;
}

ostream& AttributeInfo::toOStreamAsHTML(ostream& os) const {
  os << "<li>AttributeInfo:</li>" << endl
     << "<ul>" << endl
     << "<li>name_idx(";
  ByteClass::referenceTo(os, getAttributeNameIndex(), "ConstantPool");
  os << ")</li>" << endl
     << "<li>length(" << getAttributeLength() << ")</li>" << endl
     << "</ul>" << endl;

  return os;
}

ostream& operator<<(ostream& os, const AttributeInfo& ai) {
  return ai.toOStream(os);
}
