#ifndef BYTECLASS_HPP
#define BYTECLASS_HPP BYTECLASS_HPP

#include "CPInfo.hpp"
#include "FieldInfo.hpp"
#include "MethodInfo.hpp"
#include "AttributeInfo.hpp"
#include "JavaString.hpp"

#include <sys/types.h>

#include <vector>
#include <iostream>

using namespace std;

class ClassLoader;

class ByteClass {
private:
  friend class ClassLoader;

  u_int32_t magic;
  u_int16_t minor_version;
  u_int16_t major_version;

  vector<CPInfo*> constant_pool;

  u_int16_t access_flags;
  u_int16_t this_class;
  u_int16_t super_class;

  vector<u_int16_t> interfaces;
  vector<FieldInfo*> fields;
  vector<MethodInfo*> methods;
  vector<AttributeInfo*> attributes;

  JavaString this_class_name;
  JavaString super_class_name;

  ByteClass(const u_int32_t& = 0,
            const u_int16_t& = 0,
            const u_int16_t& = 0);
  
public:
  enum AccessFlags {
    ACC_PUBLIC       = 0x0001U,
    ACC_PRIVATE      = 0x0002U,
    ACC_PROTECTED    = 0x0004U,
    ACC_STATIC       = 0x0008U,
    ACC_FINAL        = 0x0010U,
    ACC_SUPER        = 0x0020U,
    ACC_SYNCHRONIZED = 0x0020U,
    ACC_VOLATILE     = 0x0040U,
    ACC_TRANSIENT    = 0x0080U,
    ACC_NATIVE       = 0x0100U,
    ACC_INTERFACE    = 0x0200U,
    ACC_ABSTRACT     = 0x0400U,
    ACC_STRICT       = 0x0800U
  };

  ~ByteClass();

  const u_int32_t& getMagic() const;
  const u_int16_t& getMinorVersion() const;
  const u_int16_t& getMajorVersion() const;
  const u_int16_t getConstantPoolCount() const;
  const CPInfo& getCPInfo(const u_int16_t&) const;
  const u_int16_t& getAccessFlags() const;
  const u_int16_t& getThisClass() const;
  const u_int16_t& getSuperClass() const;
  const u_int16_t getInterfacesCount() const;
  const u_int16_t& getInterface(const u_int16_t&) const;
  const u_int16_t getFieldsCount() const;
  const FieldInfo& getFieldInfo(const u_int16_t&) const;
  const u_int16_t getMethodsCount() const;
  const MethodInfo& getMethodInfo(const u_int16_t&) const;
  const u_int16_t getAttributesCount() const;
  const AttributeInfo& getAttributeInfo(const u_int16_t&) const;

  ostream& toOStreamAsHTML(ostream&) const;

  static ostream& makeReference(ostream&, const u_int16_t&, const string&);
  static ostream& referenceTo(ostream&, const u_int16_t&, const string&);

  const JavaString& getThisClassName() const;
  const JavaString& getSuperClassName() const;

};

ostream& operator<<(ostream&, const ByteClass&);

#endif
