#include "CPInfo.hpp"

#include "JavaString.hpp"

CPInfo::CPInfo(const Tag& t /*= CONSTANT_Utf8*/,
               const u_int16_t& l /*= 0*/,
               u_int8_t* i /*= 0*/)
  : tag(t),
    length(l),
    info(i) {
}

CPInfo::~CPInfo() {
  if (info) {
    delete[] info;
    info = NULL;
  }
}

void CPInfo::setTag(const Tag& t) {
  tag = t;
}

void CPInfo::setData(const u_int16_t& l, const u_int8_t* i) {
  if ((length != l) || !info) {
    // we have to realloc!
    length = l;
    if (info) {
      delete[] info;
    }
    info = new u_int8_t[length];
  }

  memcpy(info, i, length);
}

const CPInfo::Tag& CPInfo::getTag() const {
  return tag;
}

const u_int16_t& CPInfo::getLength() const {
  return length;
}

const u_int8_t* CPInfo::getInfo() const {
  return info;
}

ostream& CPInfo::toOStream(ostream& os) const {
  os << "Tag: " << getTag();

  return os;
}

ostream& CPInfo::toOStreamAsHTML(ostream& os) const {
  os << "<li>tag(" << getTag() << ")</li>" << endl;

  return os;
}

ostream& operator<<(ostream& os, const CPInfo::Tag& tag) {
    switch (tag) {
    case CPInfo::CONSTANT_Utf8:
      os << "CONSTANT_UTF8";
      break;

    case CPInfo::CONSTANT_Integer:
      os << "CONSTANT_Integer";
      break;

    case CPInfo::CONSTANT_Float:
      os << "CONSTANT_Float";
      break;

    case CPInfo::CONSTANT_Long:
      os << "CONSTANT_Long";
      break;

    case CPInfo::CONSTANT_Double:
      os << "CONSTANT_Double";
      break;

    case CPInfo::CONSTANT_Class:
      os << "CONSTANT_Class";
      break;

    case CPInfo::CONSTANT_String:
      os << "CONSTANT_String";
      break;

    case CPInfo::CONSTANT_Fieldref:
      os << "CONSTANT_Fieldref";
      break;

    case CPInfo::CONSTANT_Methodref:
      os << "CONSTANT_Methodref";
      break;

    case CPInfo::CONSTANT_InterfaceMethodref:
      os << "CONSTANT_InterfaceMethoderef";
      break;

    case CPInfo::CONSTANT_NameAndType:
      os << "CONSTANT_NameAndType";
      break;

    default:
      cerr << "Unkown ConstantPool entry tag: " << tag << endl;
      break;
    }

    return os;
}

ostream& operator<<(ostream& os, const CPInfo& cpinfo) {
  return cpinfo.toOStream(os);
}
