#ifndef CODEATTRIBUTE_HPP
#define CODEATTRIBUTE_HPP CODEATTRIBUTE_HPP

#include "AttributeInfo.hpp"

#include <vector>

using namespace std;

class CodeAttribute
  : public AttributeInfo {
public:
  class ExceptionTable {
  private:
    u_int16_t start_pc;
    u_int16_t end_pc;
    u_int16_t handler_pc;
    u_int16_t catch_type;

  public:
    ExceptionTable(const u_int16_t& start = 0,
                   const u_int16_t& end = 0,
                   const u_int16_t& handler = 0,
                   const u_int16_t& type = 0)
      : start_pc(start),
        end_pc(end),
        handler_pc(handler),
        catch_type(type) {
    }

    const u_int16_t& getStartPC()   const { return start_pc;   }
    const u_int16_t& getEndPC()     const { return end_pc;     }
    const u_int16_t& getHandlerPC() const { return handler_pc; }
    const u_int16_t& getCatchType() const { return catch_type; }
  };

private:
  friend class ClassLoader;

  u_int16_t max_stack;
  u_int16_t max_locals;
  u_int32_t code_length;
  u_int8_t* code;
  vector<ExceptionTable*> exception_table;
  vector<AttributeInfo*> attributes;

public:
  CodeAttribute(const u_int16_t& index = 0,
                const u_int16_t& mstack = 0,
                const u_int16_t& mlocals = 0,
                const u_int32_t& clength = 0,
                const u_int8_t* c = NULL);
  virtual ~CodeAttribute();

  const u_int16_t& getMaxStack() const;
  const u_int16_t& getMaxLocals() const;
  const u_int32_t& getCodeLength() const;
  const u_int8_t* getCode() const;
  u_int16_t getExceptionTableLength() const;
  const ExceptionTable& getExceptionTable(const u_int16_t&) const;
  u_int16_t getAttributesCount() const;
  const AttributeInfo& getAttributeInfo(const u_int16_t&) const;
  
  virtual ostream& toOStream(ostream&) const;
  virtual ostream& toOStreamAsHTML(ostream&) const;
};

#endif
