#include "ExceptionsAttribute.hpp"

#include "ByteClass.hpp"

ExceptionsAttribute::ExceptionsAttribute(const u_int16_t& index /*= 0*/)
  : AttributeInfo(index, 0, NULL),
    exception_index_table() {
}

ExceptionsAttribute::~ExceptionsAttribute() {
}

u_int16_t ExceptionsAttribute::getNumberOfExceptions() const {
  return exception_index_table.size();
}

const u_int16_t& ExceptionsAttribute::getExceptionIndexTable(const u_int16_t& idx) const {
  if (idx >= exception_index_table.size()) {
    cerr << "Index " << idx << " for exception_index_table out of bounds!" << endl;
    throw exception();
  }

  return exception_index_table[idx];
}

ostream& ExceptionsAttribute::toOStream(ostream& os) const {
  os << "Exceptions:\t";
  if (exception_index_table.size() > 0) {
    for (unsigned int i = 0; i < exception_index_table.size(); i++) {
      os << "idx(" << exception_index_table[i] << ")";
      if (i+1 < exception_index_table.size()) {
        os << ", ";
      }
    }
  } else {
    os << "<none>";
  }

  return os;
}

ostream& ExceptionsAttribute::toOStreamAsHTML(ostream& os) const {
  os << "<li>Exceptions:</li>" << endl
     << "<pre>";
  if (exception_index_table.size() > 0) {
    for (unsigned int i = 0; i < exception_index_table.size(); i++) {
      os << "idx(";
      ByteClass::referenceTo(os, exception_index_table[i], "ConstantPool");
      os << ")";
      if (i+1 < exception_index_table.size()) {
        os << ", ";
      }
    }
  } else {
    os << "<none>";
  }
  os << "</pre>" << endl;

  return os;
}

