#include "InnerClassesAttribute.hpp"

#include "ByteClass.hpp"

InnerClassesAttribute::InnerClassesAttribute(const u_int16_t& index /*= 0*/)
  : AttributeInfo(index, 0, NULL),
    inner_classes_index_table() {
}

InnerClassesAttribute::~InnerClassesAttribute() {
}

u_int16_t InnerClassesAttribute::getNumberOfInnerClasses() const {
  return inner_classes_index_table.size();
}

const u_int16_t& InnerClassesAttribute::getInnerClassesIndexTable(const u_int16_t& idx) const {
  if (idx >= inner_classes_index_table.size()) {
    cerr << "Index " << idx << " for inner_classes_index_table out of bounds!" << endl;
    throw exception();
  }

  return inner_classes_index_table[idx];
}

ostream& InnerClassesAttribute::toOStream(ostream& os) const {
  os << "InnerClasses:\t";
  if (inner_classes_index_table.size() > 0) {
    for (unsigned int i = 0; i < inner_classes_index_table.size(); i++) {
      os << "idx(" << inner_classes_index_table[i] << ")";
      if (i+1 < inner_classes_index_table.size()) {
        os << ", ";
      }
    }
  } else {
    os << "<none>";
  }

  return os;
}

ostream& InnerClassesAttribute::toOStreamAsHTML(ostream& os) const {
  os << "<li>InnerClasses:</li>" << endl
     << "<pre>";
  if (inner_classes_index_table.size() > 0) {
    for (unsigned int i = 0; i < inner_classes_index_table.size(); i++) {
      os << "idx(";
      ByteClass::referenceTo(os, inner_classes_index_table[i], "ConstantPool");
      os << ")";
      if (i+1 < inner_classes_index_table.size()) {
        os << ", ";
      }
    }
  } else {
    os << "<none>";
  }
  os << "</pre>" << endl;

  return os;
}

