#include "JavaString.hpp"

#include <iostream>

JavaString::JavaString() 
    : data(NULL),
      data_length(0),
      string_length(0) {
}

JavaString::JavaString(const JavaString& js) {
  *this = js;
}

JavaString::~JavaString() {
  if (data) {
    delete[] data;
    data = NULL;
  }
}

size_t JavaString::getLength() const {
  return string_length;
}

string JavaString::toSTLString() const {
  if (string_length == 0) {
    return "";
  }

  char retval[string_length];
  for (size_t i = 0; i < string_length; i++) {
    retval[i] = (char) data[i];
  }

  return string(retval, string_length);
}

JavaString JavaString::fromSTLString(const string& stl_string) {
  JavaString java_string;

  java_string.data_length = 
    java_string.string_length = 
    stl_string.length();

  java_string.data = new u_int16_t[java_string.data_length];

  for (size_t i = 0; i < java_string.data_length; i++) {
    java_string.data[i] = 0x00FF & (u_int16_t) stl_string[i];
  }

  return java_string;
}

JavaString JavaString::fromUTF8(const u_int8_t* data, const size_t& length) {
  JavaString java_string;

  java_string.data_length = length; // there are at most length characters encoded
  java_string.data = new u_int16_t[java_string.data_length];

  try {
    size_t rhead(0);
    size_t whead(0);
    while (rhead < length) {
      if (! (data[rhead] & 0x80)) { // 0x0....... ASCII 7Bit
        java_string.data[whead] = data[rhead];
        rhead++;
        whead++;
      } else {                      // not 0x0....... ASCII 7Bit
        if (data[rhead] & 0xC0) {   // 0x11......
          if (! (data[rhead] & 0xE0)) { // 0x110..... 0x10......
            if (rhead+1 >= length) {
              cerr << "Parsing error 1" << endl;
              throw exception();
            }
            u_int16_t first_bits = data[rhead] & 0x1F;
            rhead++;
            u_int16_t second_bits = data[rhead] & 0x3F;
            rhead++;
            java_string.data[whead] = (first_bits << 6) | second_bits;
            whead++;
          } else {                  // 0x1110.... 0x10...... 0x10......
            if (rhead+2 >= length) {
              cerr << "Parsing error 2" << endl;
              throw exception();
            }
            u_int16_t first_bits = data[rhead] & 0x0F;
            rhead++;
            u_int16_t second_bits = data[rhead] & 0x3F;
            rhead++;
            u_int16_t third_bits = data[rhead] & 0x3F;
            rhead++;
            java_string.data[whead] =
              (first_bits << 12) |
              (second_bits << 6) |
              third_bits;
            whead++;
          }
        }
      }
    }
    java_string.string_length = whead;
  }
  catch (...) {
//     delete java_string;
//     java_string = NULL;
  }

  return java_string;
}

JavaString& JavaString::operator=(const JavaString& js) {
  if (data) {
    delete[] data;
    data = NULL;
  }

  data_length = js.data_length;
  string_length = js.string_length;
  if (js.data) {
    data = new u_int16_t[data_length];
    memcpy(data, js.data, data_length * sizeof(u_int16_t));
  }

  return *this;
}

ostream& operator<<(ostream& os, const JavaString& java_string) {
  os << java_string.toSTLString();
  return os;
}
