#include "LineNumberTableAttribute.hpp"

LineNumberTableAttribute::LineNumberTableAttribute(const u_int16_t& index /*= 0*/)
  : AttributeInfo(index, 0, NULL),
    line_number_table() {
}

LineNumberTableAttribute::~LineNumberTableAttribute() {
}

u_int16_t LineNumberTableAttribute::getLineNumberTableLength() const {
  return line_number_table.size();
}

const LineNumberTableAttribute::LineNumberTable& LineNumberTableAttribute::getLineNumberTable(const u_int16_t& idx) const {
  if (idx >= line_number_table.size()) {
    cerr << "Index " << idx << " for line_number_table out of bounds!" << endl;
    throw exception();
  }

  return *(line_number_table[idx]);
}

ostream& LineNumberTableAttribute::toOStream(ostream& os) const {
  os << "LineNumberTable:\t";
  if (line_number_table.size() > 0) {
    for (unsigned int i = 0; i < line_number_table.size(); i++) {
      os << line_number_table[i]->getStartPC() << " -> "
         << line_number_table[i]->getLineNumber();
      if (i+1 < line_number_table.size()) {
        os << ", ";
      }
    }
  } else {
    os << "<none>" << endl;
  }

  return os;
}

ostream& LineNumberTableAttribute::toOStreamAsHTML(ostream& os) const {
  os << "<li>LineNumberTable:</li>" << endl
     << "<table border=\"2\">" << endl
     << "<tbody>" << endl
     << "<tr>" << endl
     << "<th>ByteCode</th><th>SourceCode</th>" << endl
     << "</tr>";
  for (unsigned int i = 0; i < line_number_table.size(); i++) {
    os << "<tr>" << endl
       << "<td>" << line_number_table[i]->getStartPC() << "</td>"
       << "<td>" << line_number_table[i]->getLineNumber() << "</td>" << endl
       << "</tr>";
  }
  os << "</tbody>" << endl
     << "</table>" << endl;

  return os;
}

