#include "LocalVariableTableAttribute.hpp"

#include "ByteClass.hpp"

LocalVariableTableAttribute::LocalVariableTableAttribute(const u_int16_t& index /*= 0*/)
  : AttributeInfo(index, 0, NULL),
    local_variable_table() {
}

LocalVariableTableAttribute::~LocalVariableTableAttribute() {
}

u_int16_t LocalVariableTableAttribute::getLocalVariableTableLength() const {
  return local_variable_table.size();
}

const LocalVariableTableAttribute::LocalVariableTable& LocalVariableTableAttribute::getLocalVariableTable(const u_int16_t& idx) const {
  if (idx >= local_variable_table.size()) {
    cerr << "Index " << idx << " for local_variable_table out of bounds!" << endl;
    throw exception();
  }

  return *(local_variable_table[idx]);
}

ostream& LocalVariableTableAttribute::toOStream(ostream& os) const {
  os << "LocalVariableTable:\t";
  if (local_variable_table.size() > 0) {
    for (unsigned int i = 0; i < local_variable_table.size(); i++) {
      os << "(" << local_variable_table[i]->getStartPC() << "-"
         << local_variable_table[i]->getStartPC()+local_variable_table[i]->getLength() << ") -> ("
         << local_variable_table[i]->getNameIndex() << ", "
         << local_variable_table[i]->getDescriptorIndex() << ", "
         << local_variable_table[i]->getIndex() << ")";
      if (i+1 < local_variable_table.size()) {
        os << ", ";
      }
    }
  } else {
    os << "<none>" << endl;
  }

  return os;
}

ostream& LocalVariableTableAttribute::toOStreamAsHTML(ostream& os) const {
  os << "<li>LocalVariableTable:</li>" << endl
     << "<table border=\"2\">" << endl
     << "<tbody>" << endl
     << "<tr>" << endl
     << "<th>ByteCodeRange</th><th>Name</th><th>Descriptor</th><th>Index</th>" << endl
     << "</tr>";
  for (unsigned int i = 0; i < local_variable_table.size(); i++) {
    os << "<tr>" << endl
       << "<td>" << local_variable_table[i]->getStartPC() << "-" << local_variable_table[i]->getStartPC()+local_variable_table[i]->getLength() << "</td>"
       << "<td>idx(";
    ByteClass::referenceTo(os, local_variable_table[i]->getNameIndex(), "ConstantPool");
    os << ")</td>" << endl
       << "<td>idx(";
    ByteClass::referenceTo(os, local_variable_table[i]->getDescriptorIndex(), "ConstantPool");
    os << ")</td>" << endl
       << "<td>" << local_variable_table[i]->getIndex() << "</td>" << endl
       << "</tr>";
  }
  os << "</tbody>" << endl
     << "</table>" << endl;

  return os;
}

