#include "MethodInfo.hpp"

#include "ByteClass.hpp"

#include <list>

list<string> getMethodAccessFlags(const u_int16_t& access_flags) {
  list<string> access_flags_list;
  if (access_flags & ByteClass::ACC_PUBLIC) {
    access_flags_list.push_back("public");
  }
  if (access_flags & ByteClass::ACC_PRIVATE) {
    access_flags_list.push_back("private");
  }
  if (access_flags & ByteClass::ACC_PROTECTED) {
    access_flags_list.push_back("protected");
  }
  if (access_flags & ByteClass::ACC_STATIC) {
    access_flags_list.push_back("static");
  }
  if (access_flags & ByteClass::ACC_FINAL) {
    access_flags_list.push_back("final");
  }
  if (access_flags & ByteClass::ACC_SYNCHRONIZED) {
    access_flags_list.push_back("synchronized");
  }
  if (access_flags & ByteClass::ACC_NATIVE) {
    access_flags_list.push_back("native");
  }
  if (access_flags & ByteClass::ACC_ABSTRACT) {
    access_flags_list.push_back("abstract");
  }
  if (access_flags & ByteClass::ACC_STRICT) {
    access_flags_list.push_back("strictfp");
  }

  return access_flags_list;
}

ostream& MethodInfo::toOStream(ostream& os) const {
  os << "MethodInfo:"
     << "\tAccessFlags: ";
  list<string> afl(getMethodAccessFlags(getAccessFlags()));
  for (list<string>::const_iterator af = afl.begin(); af != afl.end(); af++) {
    if (af != afl.begin()) {
      os << ", ";
    }
    os << *af;
  }
  os << "\tname_idx(" << getNameIndex() << ")"
     << "\tdescriptor_idx(" << getDescriptorIndex() << ")";
  if (getAttributesCount() > 0) {
    os << "\nAttributes:\n";
    for (unsigned int i = 0; i < getAttributesCount(); i++) {
      os << getAttribute(i);
      if (i+1 != getAttributesCount()) {
        os << "\n";
      }
    }
  }

  return os;
}

ostream& MethodInfo::toOStreamAsHTML(ostream& os) const {
  os << "<li>MethodInfo:</li>" << endl
     << "<ul>"
     << "<li>AccessFlags: ";
  list<string> afl(getMethodAccessFlags(getAccessFlags()));
  for (list<string>::const_iterator af = afl.begin(); af != afl.end(); af++) {
    if (af != afl.begin()) {
      os << ", ";
    }
    os << *af;
  }
  os << "</li>" << endl
     << "<li>name_idx(";
  ByteClass::referenceTo(os, getNameIndex(), "ConstantPool");
  os << ")</li>" << endl
     << "<li>descriptor_idx(";
  ByteClass::referenceTo(os, getDescriptorIndex(), "ConstnantPool");
  os << ")</li>" << endl;
  if (getAttributesCount() > 0) {
    os << "<li>Attributes:</li>" << endl
       << "<ul>" << endl;
    for (unsigned int i = 0; i < getAttributesCount(); i++) {
      getAttribute(i).toOStreamAsHTML(os);
    }
    os << "</ul>" << endl;
  }
  os << "</ul>" << endl;

  return os;
}
