#include <getopt.h>

#include <iostream>

#include "ClassLoader.hpp"

using namespace std;

void usage(const char* name) {
  cerr << "This is ClassLoader, V0.1" << endl
       << endl
       << "Usage: " << name << " [options] <java.class>" << endl
       << "Options:" << endl
       << "  -H --html:   Output as HTML." << endl
       << "  -o --output: Specify output file." << endl
       << "  -h --help:   This message." << endl;
}


int main(int argc, char* argv[]) {
  // we need a file to parse
  string classfile("");
  string outfile("");
  bool html_output(false);

  // get options with getopt
  {
    const char* optstring = "Ho:h";
    // no_argument, required_argument, optional_argument are defined
    // in getopt.h
    struct option longopts[] = {
      {"html",   no_argument,       NULL, 'H'},
      {"output", required_argument, NULL, 'o'},
      {"help",   no_argument,       NULL, 'h'},
      {NULL,     0,                 NULL, 0  } /* last row must be zero */
    };
  
    /* some parsing variables */
    int ch, optionindex;

    while(EOF != (ch=getopt_long(argc, argv, optstring, longopts, &optionindex))){
      // which option did we get?
      switch (ch) {
      case 'H':
        html_output = true;
        break;

      case 'o':
        outfile = string(optarg);
        break;

      case 'h':
        // fallthrough
      default:
        usage(argv[0]);
        return 1;
      }
    }

    // get non-option argument, the file to parse
    if (optind < argc) {
      classfile = string(argv[optind]);  
    } else {
      usage(argv[0]);
      return 1;
    }
  }

  ostream* osptr(NULL);
  ofstream outfstream;
  if (outfile.empty()) {
    osptr = &cout;
  } else {
    outfstream.open(outfile.c_str());
    if (outfstream.is_open()) {
      osptr = &outfstream;
    } else {
      cerr << "Error while opening \"" << outfile << "\" for writing!" << endl;
      return 1;
    }
  }
  ostream& os(*osptr);

  // no parse filename
  try {
    ClassLoader cl;
    ByteClass* c(cl.load(classfile));

    if (html_output) {
      os << "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">" << endl
         << "<html>" << endl
         << "<head>" << endl
         << "<title>ClassLoader: " << classfile << "</title>" << endl
         << "</head>" << endl
         << endl
         << "<body>" << endl
         << "<h1>ClassLoader: " << classfile << "</h1>" << endl;
      c->toOStreamAsHTML(os);
      os << "</body>" << endl
         << "</html>" << endl;
    } else {
      os << *c << endl;
    }

    delete c;
  } catch (exception e) {
    cerr << "Something went wrong!" << endl;
  }

  return 0;
}
